@@ -16,7 +16,18 @@ Puppet::Type.type(:keystone_user).provide(
     @property_flush = {}
   end
 
+  def self.do_not_manage
+    @do_not_manage
+  end
+
+  def self.do_not_manage=(value)
+    @do_not_manage = value
+  end
+
   def create
+    if self.class.do_not_manage
+      fail("Not managing Keystone_user[#{@resource[:name]}] due to earlier Keystone API failures.")
+    end
     user_name, user_domain = resource[:name], resource[:domain]
     properties = [user_name]
     if resource[:enabled] == :true
@@ -41,6 +52,9 @@ Puppet::Type.type(:keystone_user).provide(
   end
 
   def destroy
+    if self.class.do_not_manage
+      fail("Not managing Keystone_user[#{@resource[:name]}] due to earlier Keystone API failures.")
+    end
     self.class.request('user', 'delete', id)
     @property_hash.clear
   end
@@ -74,10 +88,16 @@ Puppet::Type.type(:keystone_user).provide(
   end
 
   def enabled=(value)
+    if self.class.do_not_manage
+      fail("Not managing Keystone_user[#{@resource[:name]}] due to earlier Keystone API failures.")
+    end
     @property_flush[:enabled] = value
   end
 
   def email=(value)
+    if self.class.do_not_manage
+      fail("Not managing Keystone_user[#{@resource[:name]}] due to earlier Keystone API failures.")
+    end
     @property_flush[:email] = value
   end
 
@@ -109,6 +129,9 @@ Puppet::Type.type(:keystone_user).provide(
         # last chance - try a domain scoped token
         credentials.domain_name = domain
       end
+
+      credentials.identity_api_version = '2' if credentials.auth_url =~ /v2\.0\/?$/
+
       begin
         token = Puppet::Provider::Openstack.request('token', 'issue', ['--format', 'value'], credentials)
       rescue Puppet::Error::OpenstackUnauthorizedError
@@ -121,6 +144,9 @@ Puppet::Type.type(:keystone_user).provide(
   end
 
   def password=(value)
+    if self.class.do_not_manage
+      fail("Not managing Keystone_user[#{@resource[:name]}] due to earlier Keystone API failures.")
+    end
     @property_flush[:password] = value
   end
 
@@ -129,6 +155,9 @@ Puppet::Type.type(:keystone_user).provide(
   end
 
   def replace_password=(value)
+    if self.class.do_not_manage
+      fail("Not managing Keystone_user[#{@resource[:name]}] due to earlier Keystone API failures.")
+    end
     @property_flush[:replace_password] = value
   end
 
@@ -144,8 +173,9 @@ Puppet::Type.type(:keystone_user).provide(
     if default_domain_changed
       warning(default_domain_deprecation_message)
     end
+    self.do_not_manage = true
     users = request('user', 'list', ['--long'])
-    users.collect do |user|
+    list = users.collect do |user|
       domain_name = domain_name_from_id(user[:domain])
       new(
         :name        => resource_to_name(domain_name, user[:name]),
@@ -159,6 +189,8 @@ Puppet::Type.type(:keystone_user).provide(
         :id          => user[:id]
       )
     end
+    self.do_not_manage = false
+    list
   end
 
   def self.prefetch(resources)
