@@ -15,7 +15,7 @@ describe 'keystone::logging' do
       :logging_exception_prefix => '%(asctime)s.%(msecs)03d %(process)d TRACE %(name)s %(instance)s',
       :log_config_append => '/etc/keystone/logging.conf',
       :publish_errors => true,
-      :default_log_levels => { 
+      :default_log_levels => {
         'amqp' => 'WARN', 'amqplib' => 'WARN', 'boto' => 'WARN',
         'qpid' => 'WARN', 'sqlalchemy' => 'WARN', 'suds' => 'INFO',
         'iso8601' => 'WARN',
@@ -36,12 +36,12 @@ describe 'keystone::logging' do
 
   shared_examples 'basic default logging settings' do
     it 'configures glance logging settins with default values' do
-      is_expected.to contain_keystone_config('DEFAULT/use_syslog').with(:value => 'false')
-      is_expected.to contain_keystone_config('DEFAULT/use_stderr').with(:value => 'true')
+      is_expected.to contain_keystone_config('DEFAULT/use_syslog').with(:value => '<SERVICE DEFAULT>')
+      is_expected.to contain_keystone_config('DEFAULT/use_stderr').with(:value => '<SERVICE DEFAULT>')
       is_expected.to contain_keystone_config('DEFAULT/log_dir').with(:value => '/var/log/keystone')
-      is_expected.to contain_keystone_config('DEFAULT/log_file').with(:ensure => :absent)
-      is_expected.to contain_keystone_config('DEFAULT/verbose').with(:value => 'false')
-      is_expected.to contain_keystone_config('DEFAULT/debug').with(:value => 'false')
+      is_expected.to contain_keystone_config('DEFAULT/log_file').with(:value => '<SERVICE DEFAULT>')
+      is_expected.to contain_keystone_config('DEFAULT/verbose').with(:value => '<SERVICE DEFAULT>')
+      is_expected.to contain_keystone_config('DEFAULT/debug').with(:value => '<SERVICE DEFAULT>')
     end
   end
 
@@ -123,13 +123,13 @@ describe 'keystone::logging' do
      :default_log_levels, :fatal_deprecations,
      :instance_format, :instance_uuid_format,
      :log_date_format, ].each { |param|
-        it { is_expected.to contain_keystone_config("DEFAULT/#{param}").with_ensure('absent') }
+        it { is_expected.to contain_keystone_config("DEFAULT/#{param}").with_value('<SERVICE DEFAULT>') }
       }
   end
 
   context 'on Debian platforms' do
     let :facts do
-      { :osfamily => 'Debian' }
+      @default_facts.merge({ :osfamily => 'Debian' })
     end
 
     it_configures 'keystone-logging'
@@ -137,7 +137,7 @@ describe 'keystone::logging' do
 
   context 'on RedHat platforms' do
     let :facts do
-      { :osfamily => 'RedHat' }
+      @default_facts.merge({ :osfamily => 'RedHat' })
     end
 
     it_configures 'keystone-logging'
