@@ -1,3 +1,16 @@
+require 'spec_helper'
+# this hack is required for now to ensure that the path is set up correctly
+# to retrive the parent provider
+$LOAD_PATH.push(
+  File.join(
+    File.dirname(__FILE__),
+    '..',
+    '..',
+    'fixtures',
+    'modules',
+    'inifile',
+    'lib')
+)
 require 'puppet'
 require 'puppet/type/keystone_config'
 
